"""
From QGIS 3.28, QgsProject.instance() is unusually long (up to 0.5s). To avoid performance issues,
QgsProject's current instance is cached and updated when necessary.
"""

from typing import Optional
from qgis._core import QgsProject


_qgis_project_cache: Optional[QgsProject] = None

def qgis_project_cache() -> QgsProject:
    """
    Get QgsProject current instance

    :returns: QgsProject instance
    :rtype: QgsProject
    """
    global _qgis_project_cache
    if _qgis_project_cache is None:
        _qgis_project_cache = QgsProject().instance()
    return _qgis_project_cache

def update_qgis_project_cache(qgis_project: QgsProject):
    """
    Update QgsProject instance.

    :param qgis_project: The current QgsProject instance
    :type qgis_project: QgsProject
    """
    global _qgis_project_cache
    _qgis_project_cache = qgis_project