from typing import Optional, List, Dict
import numpy as np
from landsklim.lisdqs.map_layer import RasterLayer


class PolygonsDefinition:
    """
    Class containing data about polygons
    """

    def __init__(self, polygons_layer: Optional[RasterLayer], polygons_definition: Optional[List[List[int]]], polygons_connectedness: List[List[int]], polygons_centroids: Optional[np.ndarray]):
        self.__polygons_layer: Optional[RasterLayer] = polygons_layer
        self.__polygons_definition: Optional[List[List[int]]] = polygons_definition
        self.__polygons_connectedness: Optional[List[List[int]]] = polygons_connectedness
        self.__polygons_centroids: Optional[np.ndarray] = polygons_centroids

    def set_polygons(self, polygons_layer: Optional[RasterLayer], polygons_definition: Optional[List[List[int]]], polygons_connectedness: List[List[int]], polygons_centroids: Optional[np.ndarray]):
        """
        :param polygons_layer: Polygons layer
        :type polygons_layer: Optional[RasterLayer]

        :param polygons_definition: Polygons definition
        :type polygons_definition: Optional[List[List[int]]]

        :param polygons_connectedness: Connected polygons for each polygons
        :type polygons_connectedness: List[List[int]]

        :param polygons_centroids: Centroid of each polygons
        :type polygons_centroids: Optional[np.ndarray]

        """
        self.__polygons_layer = polygons_layer
        self.__polygons_definition = polygons_definition
        self.__polygons_connectedness = polygons_connectedness
        self.__polygons_centroids = polygons_centroids

    def set_polygon_layer(self, polygons_layer: RasterLayer):
        self.__polygons_layer = polygons_layer

    def polygons_layer(self) -> Optional[RasterLayer]:
        """
        :returns: RasterLayer of polygons
        :rtype: RasterLayer
        """
        return self.__polygons_layer

    def polygons_definition(self) -> Optional[List[List[int]]]:
        """
        :returns: List[List[int]] of shape (*polygons_count*, n) : Definition of each polygon,
            i.e. the stations included on the station's neighborhood
        :rtype: Optional[List[List[int]]]
        """
        return self.__polygons_definition

    def polygons_connectedness(self) -> List[List[int]]:
        """
        :returns: List[List[int]] of connectedness : List of each neighbors polygons
        :rtype: List[List[int]]
        """
        return self.__polygons_connectedness

    def polygons_centroids(self) -> Optional[np.ndarray]:
        """
        :returns: np.ndarray of center of gravity : List of each polygons' center of gravity
        :rtype: Optional[np.ndarray]
        """
        return self.__polygons_centroids

    def polygons_count(self) -> int:
        return len(self.__polygons_definition) if self.__polygons_definition is not None else 1

    def to_json(self) -> Dict:
        state_dict: Dict = self.__dict__.copy()
        return state_dict
