from typing import List
import os

from qgis.core import QgsRasterLayer, QgsProject
from qgis import processing

from landsklim.lisdqs.regressor import Regressor
from landsklim.lisdqs.landsklim_constants import LAYER_TYPE_PATH, LandsklimLayerType


class RegressorEncasement(Regressor):

    def __init__(self, windows: int, polynomial_degree: int):
        super().__init__(windows, polynomial_degree, [])

    @staticmethod
    def class_name() -> str:
        return "REGRESSOR_ENCASEMENT"

    @staticmethod
    def min_window() -> int:
        return 3

    def prefix(self) -> str:
        return "encai"

    def name(self) -> str:
        return Regressor.tr("Encasement")

    def compute(self, source_raster: QgsRasterLayer):
        raster_path, exists = self._create_path()

        project_path = self.project_path()

        if not exists:
            params = {
                'INPUT': source_raster,
                'INPUT_WINDOW': self.get_windows(),
                'INPUT_CUSTOM_NO_DATA': None,
                'ADDITIONAL_VARIABLES_FOLDER': os.path.join(project_path, LAYER_TYPE_PATH[LandsklimLayerType.AdditionalLayer]),
                'OUTPUT': raster_path
            }
            processing.run("landsklim:encasement", params)
